/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.vew.sim.sim;

import java.util.ArrayList;
import java.util.Collection;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.application.StandardApplication;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Applikationsklasse zum Erzeugen von Simulationsstrecken
 * 
 * @author Dambach Werke GmbH
 * @version $Revision: 1.1 $ / $Date: 2008/10/31 13:30:32 $ / ($Author: Sans $)
 */
public class TestSimVar implements Runnable, StandardApplication, ClientSenderInterface 
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private Debug _debug;

  /**
   * -konfigObjekt von Kommandozeile
   */
  private String _pidKonfigObjekt = null;
  
  /**
   * -simulationsStrecke von Kommandozeile
   */
  private String _pidSimulationsStrecke = null;
  
  /**
   * -clean von Kommandozeile
   */
  private boolean _clean = false;
  
  static final String MENGE = "Simulationen";
  
  /**
   * Prefix fr Simulationnamen
   */
  private String _namePrefix = "Simulation-TestSimVar-#";
  
  private ClientDavInterface _connection;
  
  private ConfigurationObject _konfigObjekt;
  
  private SystemObject _simulationsStrecke;
  
  private boolean _warten = false;
  
        
  /**
   * Default- Konstruktor.<br>
   * Wird typischerweise von <code>main</code> aufgerufen
   */
  public TestSimVar()
  {
    try
    {
      Thread.sleep(100);
    }
    catch (InterruptedException e)
    {
    }
  }
  
  /**
   * berschriebene Methode von <code>StandardApplication</code>, 
   * die die speziellen Startparameter auswertet
   * 
   * @param argumentList
   *  siehe sys.funclib.application.StandardApplication#parseArguments(sys.funclib.ArgumentList)
   */
  public void parseArguments(ArgumentList argumentList)
  {
    _debug = Debug.getLogger();
   
    _clean = argumentList.hasArgument("-clean");
    
    if (!_clean)
    {
      _pidSimulationsStrecke = argumentList.fetchArgument("-simulationsStrecke=").asNonEmptyString();
    }  
    
    if (argumentList.hasArgument("-konfigObjekt"))
    {  
      _pidKonfigObjekt = argumentList.fetchArgument("-konfigObjekt=").asNonEmptyString();
    }
    
    argumentList.fetchUnusedArguments();
  }
    
  /**
   * berschriebene Methode von StandardApplication, die die Initialisierung durchfhrt
   * 
   * @param connection
   * siehe sys.funclib.application.StandardApplication#initialize(de.bsvrz.dav.daf.main.ClientDavInterface)
   */
  public void initialize(ClientDavInterface connection)
  {
    _connection = connection;
    
    if (_pidKonfigObjekt != null)
    {
      _konfigObjekt = (ConfigurationObject)connection.getDataModel().getObject(_pidKonfigObjekt); 
    }
    else
    {
      _konfigObjekt = connection.getLocalConfigurationAuthority();
    }
    
    if (!_clean)
    {  
      _simulationsStrecke = connection.getDataModel().getObject(_pidSimulationsStrecke);
    }  
    
    new Thread(this).start();
  }
  
  /*
   * (Kein Javadoc)
   * @see de.bsvrz.dav.daf.main.ClientSenderInterface#dataRequest(de.bsvrz.dav.daf.main.config.SystemObject, de.bsvrz.dav.daf.main.DataDescription, byte)
   */
  public void dataRequest(SystemObject object, DataDescription dataDescription, byte state)
  {
    _debug.finer("dataRequest() fr " + object.getPid() + " - Sender state: " + state);
    if (_warten && state == ClientSenderInterface.START_SENDING)
    {
      synchronized(this)
      {
        _warten = false;
        notify();
      }
    }
  }

  /*
   * (Kein Javadoc)
   * @see de.bsvrz.dav.daf.main.ClientSenderInterface#isRequestSupported(de.bsvrz.dav.daf.main.config.SystemObject, de.bsvrz.dav.daf.main.DataDescription)
   */
  public boolean isRequestSupported(SystemObject so, DataDescription dd)
  {
    return true;
  }
  
  public void run()
  {
    boolean angemeldet = false;
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.simulationVerwaltungAnfrage");
    Aspect asp = _connection.getDataModel().getAspect("asp.anfrage");
    DataDescription datenBeschreibung = new DataDescription(atg, asp);
   
    try
    {
    
      try
      {
        _connection.subscribeSender(this, _konfigObjekt, datenBeschreibung, SenderRole.sender());
        angemeldet = true;
      }
      catch(OneSubscriptionPerSendData e)
      {
        // angemeldet bleibt false
      }
      
      if (!_clean)
      {
        _debug.info("Versuch, 1000 Simulationsobjekte " + _namePrefix + "1 bis " + 
            _namePrefix + "1000 zu erzeugen");
        
//        ResultData[] resultData = new ResultData[1000];
//        
//        for (int i = 0; i < 1000; i += 1)
//        {
//          Data data = _connection.createData(_connection.getDataModel().getAttributeGroup("atg.simulationVerwaltungAnfrage"));
//          data.getItem("AbsenderId").asReferenceValue().setSystemObject(_connection.getLocalApplicationObject());
//          data.getItem("AbsenderZeichen").asTextValue().setText("n/a");
//          Array array = data.getArray("ErzeugeSimulationAnfrage");
//          array.setLength(1);
//          Data entry = array.getItem(0);
//          entry.getItem("Name").asTextValue().setText(_namePrefix + (i + 1));
//          entry.getItem("ReferenzSimulationsStrecke").asReferenceValue().setSystemObject(_simulationsStrecke);
//          entry.getItem("SimulationsArt").asTextValue().setText("Online");
//          
//          resultData[i] = new ResultData(_konfigObjekt, datenBeschreibung, _connection.getTime(), data);
//        }
//        
//        _debug.info("Sende " + resultData.length +  " Erzeuge-Anfragen");
        
        Data data = _connection.createData(_connection.getDataModel().getAttributeGroup("atg.simulationVerwaltungAnfrage"));
        data.getItem("AbsenderId").asReferenceValue().setSystemObject(_connection.getLocalApplicationObject());
        data.getItem("AbsenderZeichen").asTextValue().setText("n/a");
        Array array = data.getArray("ErzeugeSimulationAnfrage");
        array.setLength(1000);
        
        for (int i = 0; i < 1000; i += 1)
        {
          Data entry = array.getItem(i);
          entry.getItem("Name").asTextValue().setText(_namePrefix + (i + 1));
          entry.getItem("ReferenzSimulationsStrecke").asReferenceValue().setSystemObject(_simulationsStrecke);
          entry.getItem("SimulationsArt").asTextValue().setText("Online");
        }
        
        send(new ResultData[] { new ResultData(_konfigObjekt, datenBeschreibung, _connection.getTime(), data) });
      }
      else
      {
        _debug.info("Lsche alle Simulationsobjekte von " + _konfigObjekt.getName() + 
            ", deren Name mit \"" + _namePrefix + "\" beginnt");
  
        MutableSet menge = _konfigObjekt.getMutableSet(MENGE);
        Collection<SystemObject> zuLoeschen = new ArrayList<SystemObject>();
        
//      Collection<ResultData> result = new ArrayList<ResultData>();
//        
//      for (SystemObject obj: menge.getElements())
//      {
//        if (obj.isValid() && obj.getName().startsWith(_namePrefix))
//        {
//          Data data = _connection.createData(_connection.getDataModel().getAttributeGroup("atg.simulationVerwaltungAnfrage"));
//          data.getItem("AbsenderId").asReferenceValue().setSystemObject(_connection.getLocalApplicationObject());
//          data.getItem("AbsenderZeichen").asTextValue().setText("n/a");
//          Array array = data.getArray("LoescheSimulationAnfrage");
//          array.setLength(1);
//          Data entry = array.getItem(0);
//          entry.getItem("ReferenzZuLoeschendeSimulation").asReferenceValue().setSystemObject(obj);
//          
//          result.add(new ResultData(_konfigObjekt, datenBeschreibung, _connection.getTime(), data));
//          zuLoeschen.add(obj);
//        }
//      }
//      
//      if (!result.isEmpty())
//      {
//        bringeInZustandGeloescht(zuLoeschen.toArray(new SystemObject[0]));
//        _connection.sleep(250);
//        _debug.info("Sende " + result.size() +  " Lsch-Anfragen");
//        _connection.sleep(250);
//        send(result.toArray(new ResultData[0]));
//      }
//      else
//      {
//        _debug.info("Keine entsprechenden Simulationsobjekte vorhanden fr " + _konfigObjekt.getName());
//      }
        
        Data data = _connection.createData(_connection.getDataModel().getAttributeGroup("atg.simulationVerwaltungAnfrage"));
        data.getItem("AbsenderId").asReferenceValue().setSystemObject(_connection.getLocalApplicationObject());
        data.getItem("AbsenderZeichen").asTextValue().setText("n/a");
        Array array = data.getArray("LoescheSimulationAnfrage");
        
        for (SystemObject obj: menge.getElements())
        {
          if (obj.isValid() && obj.getName().startsWith(_namePrefix))
          {
            int index = array.getLength();
            array.setLength(index + 1);
            array.getItem(index).getItem("ReferenzZuLoeschendeSimulation").asReferenceValue().setSystemObject(obj);
            zuLoeschen.add(obj);
          }
        }
        
        if (!zuLoeschen.isEmpty())
        {
          _debug.info("Sende " + zuLoeschen.size() +  " Lsch-Anfragen");
          bringeInZustandGeloescht(zuLoeschen.toArray(new SystemObject[0]));
          _connection.sleep(500);
          send(new ResultData[] { new ResultData(_konfigObjekt, datenBeschreibung, _connection.getTime(), data) });
        }
        else
        {
          _debug.info("Keine entsprechenden Simulationsobjekte vorhanden fr " + _konfigObjekt.getName());
        }
        
      }
    }
    catch (Exception e)
    {
      _debug.error("Fehler", e);
    }
    finally
    {
      if (angemeldet)
      {
        _connection.unsubscribeSender(this, _konfigObjekt, datenBeschreibung);
      }
      _debug.info("Programmende");
      System.exit(0);
    }  
  }

  private void bringeInZustandGeloescht(SystemObject[] simulationen)
    throws SendSubscriptionNotConfirmed
  {
    boolean angemeldet = false;
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.simulationsSteuerungOnline");
    Aspect asp = _connection.getDataModel().getAspect("asp.vorgabe");
    DataDescription datenBeschreibung = new DataDescription(atg, asp);
    
    try
    {
    
      try
      {
        _connection.subscribeSender(this, simulationen, datenBeschreibung, SenderRole.sender());
        angemeldet = true;
      }
      catch(OneSubscriptionPerSendData e)
      {
        // angemeldet bleibt false
      }
      
      ResultData[] resultData = new ResultData[simulationen.length];
      
      for (int i = 0; i < simulationen.length; i += 1)
      {  
        Data data = _connection.createData(atg);
        data.getItem("SimulationsZustand").asTextValue().setText("gelscht");
      
        resultData[i] = new ResultData(simulationen[i], datenBeschreibung, _connection.getTime(), data);
      }
      
      send(resultData);
    }  
    finally
    {
      if (angemeldet)
      {
        _connection.unsubscribeSender(this, simulationen, datenBeschreibung);
      }
    }  
  }
  
  private void send(ResultData[] resultData) throws SendSubscriptionNotConfirmed
  {
    try
    {
      _connection.sendData(resultData);
    }
    catch (SendSubscriptionNotConfirmed e)
    {
      _debug.info("Warten auf Sendesteuerung ...");
      synchronized(this)
      {
        _warten = true;
        while (_warten)
        {
          try
          {
            wait();
          }
          catch (InterruptedException ex)
          {
            // tue nichts
          }
        }  
      }
      _connection.sendData(resultData);
    }
  }
  
  /**
   * Programmeinstieg
   */
  public static void main(String[] arguments)
  {
    StandardApplicationRunner.run(new TestSimVar(), arguments);
  }
  
}
