/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

 package de.bsvrz.vew.sim.sim.simulation;

 /**
  * Implementierung eines endlichen Automaten
  * 
  * @version $Revision: 1.3 $ / $Date: 2008/10/31 13:30:30 $ / ($Author: Sans $)
  *
  * @author Dambach-Werke GmbH
  * @author Stefan Sans
  */    
public class EndlicherAutomat 
{
  /**
   * Anzahl der Zustnde dieses endlichen Automaten
   */    
  protected int _anzahlZustaende = 0;
  
  /**
   * Momentaner Zustand dieses endlichen Automaten
   */
  protected Zustand _momentZustand = null;
  
  /**
   * In diesem Feld werden alle mgliche Zustnde dieses 
   * endlichen Automaten gespeichert 
   */
  protected Zustand[] _zustaende = null;
  
  
  /**
   *  Erzeugt einen neuen endlichen Auotmaten
   */ 
  public EndlicherAutomat() 
  {
      this(32);
  }
  
  /**
   *  Erzeugt einen neuen endlichen Auotmaten mit einer 
   *  maximalen Anzahl von Zustnden
   *  
   *  @param maxZustaende maximale Anzahl Zustnde
   */    
  public EndlicherAutomat(int maxZustaende) 
  {
    _zustaende = new Zustand[maxZustaende];
    //numStates = maxStates;
  }
  
  /**
   *  Erzeugt einen neuen endlichen Auotmaten mit einem 
   *  initialen Zustand
   *  
   *  @param initialZustand der initiale Zustand
   */
  public EndlicherAutomat(Zustand initialZustand) 
  {
    this();
    hinzufuegenZustand(initialZustand);
    setZustand(initialZustand);
  }
  
  /**
   *  Erzeugt einen neuen endlichen Auotmaten mit einer 
   *  maximalen Anzahl von Zustnden und einem 
   *  initialen Zustand
   *  
   *  @param maxZustaende maximale Anzahl Zustnde
   *  @param initialZustand der initiale Zustand
   */    
  public EndlicherAutomat(int maxZustaende, Zustand initialZustand)
  {
    this(maxZustaende);
    hinzufuegenZustand(initialZustand);
    setZustand(initialZustand);
  }
  
  /**
   * Bestimmt den momentanen Zustand dieses endlichen Automaten
   * @return den momentanen Zustand
   */    
  public Zustand getMomentZustand() 
  {
    return _momentZustand;
  }
  
  /**
   * Fgt diesem endlichen Automaten einen neuen Zustand hinzu.
   * 
   * @param zustand der neue Zustand
   * @return die resultierende ANzahl von Zustnden
   */    
  public int hinzufuegenZustand(Zustand zustand) 
  {
    //System.out.println(fsmStates.length + " : " + numStates);
    
    if (_anzahlZustaende == _zustaende.length)
        throw new IllegalArgumentException("Maximale Anzahl Zustnde erreicht");
    
    _zustaende[_anzahlZustaende++] = zustand;
    
    return _anzahlZustaende;
  }
  
  /**
   * Setze den momentanen Zustand dieses endlichen Automaten
   *
   * @param zustand der zu setzende momentane Zustand
   */
  public void setZustand(Zustand zustand)
  {
    _momentZustand = zustand;
  }
  
  /**
   * Fgt diesem endlichen Automaten ein Feld von Zustnden hinzu
   * 
   * @param zustaende das hinzuzufgende Feld von Zustnden
   * @return die resultierende Anzahl Zustnde
   */    
  public int hinzufuegenZustaende(Zustand[] zustaende) 
  {
    _anzahlZustaende = zustaende.length;
    _zustaende = zustaende;
      
      return _anzahlZustaende;
  }
  
  /**
   * Lsche alle Zustnde dieses endlichen Automaten 
   */    
  public void loescheAlleZustaende() 
  {
    // Keep the array allocated, but remove the
    // states.
    for (int i=0; i<_zustaende.length; i++)
      _zustaende[i] = null;
  }
  
  /**
   * Fgt diesem endlichen Automaten einen neuen Zustands-bergang hinzu.
   * Das Aufrufen dieser Methode ist aquivalent zum hinzufgen eines neuen 
   * bergangs direkt zum Anfangszustand
   * @param start der Anfangszustand des neuen bergangs
   * @param uebergang die Id des neuen bergangs
   * @param end der Endzustand des neuen bergangs
   */
  public void hinzufuegenUebergang(Zustand start, int uebergang, Zustand end) 
  {
    start.hinzufuegenUebergang(uebergang, end);
  }
  
  /**
   * Realisiert den gegebenen bergang fr diesen endlichen Automaten.
   * 
   * @param uebergang Id des bergangs, der realisiert werden soll
   * @return den neuen Zustand ded endlcine Automaten
   * @throws IllegalStateException falls der gegebene bergang fr diesen 
   * endlchen Automaten nicht definiert ist
   */    
  public Zustand realisiereUebergang(int uebergang)
    throws IllegalStateException
  {
    Zustand neuZustand = _momentZustand.realisiereUebergang(uebergang);
    
    if (neuZustand == null)
        throw new IllegalStateException("bergang zum Zielzustand nicht definiert");
    
    _momentZustand = neuZustand;
    
    return _momentZustand;
  }
  
  /**
   * Berechnet fr gegebenen Anfang-und Endzustand die Id des bergangs.
   *  
   * @param start der Anfangs-Zustand des zu berechnenden bergangs
   * @param ende der End-Zustand des zu berechnenden bergangs
   * @return die Id des bergangs Id des bergangs von start zu ende
   * @throws UnsupportedOperationException falls ein bergang  von start 
   * zu ende fr diese Instanz nicht definiert ist
   */
  public int berechneUebergang(Zustand start, Zustand ende)
    throws UnsupportedOperationException 
  {
    return start.berechneUebergang(ende);
  }
  
  /**
   * Bestimme den gegebenen Zustand dieses endlichen Automaten
   * 
   * @param i Index des zurckzugebenden Zustands
   * @return gegebener Zustand
   */    
  public Zustand getZustand(int i) 
  {
    return _zustaende[i];
  }
       
  /**
   * Bestimme die Anzahl der Zustnde dieses endlichen Automaten
   * 
   * @return die Anzahl Zustnde
   */
  public int getAnzahlZustaende()
  {
    return _anzahlZustaende;
  }
   
}
